/****************************************************************************

    PROGRAM: DLLMusic.c

    PURPOSE: DLLMusic template for Sound Blaster applications

    FUNCTIONS:

    WinMain () - calls initialization function, processes message loop
    InitApplication () - initializes window data and registers window
    InitInstance () - saves instance handle and creates main window
    MainWndProc () - processes messages
    About () - processes messages for "About" dialog box

    COMMENTS:

        Windows can have several copies of your application running at the
        same time. The variable hInst keeps track of which instance this
        application is so that processing will be to the correct window.

****************************************************************************/

#include "windows.h"            /* required for all Windows applications */
#include "dllmusic.h"           /* specific to this program              */
#include <sndblst.h>

WORD    wDLLVersion;            /* verion number of DLL                  */
WORD    wSoundBlasterMsg;       /* Sound Blaster message                 */
LPSTR   lpCardName;             /* name of Sound Blaster card            */
HANDLE  hInst;                  /* current instance                      */
HANDLE  hMusicMem;              /* handle of allocated memory            */
LPSTR   lpBuffer;               /* buffer to hold voice data             */
HWND    hWnd;                   /* main window handle.                   */
int     wTempoValue=100;        /* tempo value                           */
int     wTransposeValue=0;      /* transpose value                       */
WORD    wMusicStatus=0;         /* music status                          */

/****************************************************************************

    FUNCTION: WinMain (HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point 
        for the program.  This function calls the application initialization 
        routine, if no other instance of the program is running, and always 
        calls the instance initialization routine.  It then executes a message 
        retrieval and dispatch loop that is the top-level control structure 
        for the remainder of execution. The loop is terminated when a WM_QUIT
        message is received, at which time this function exits the application 
        instance by returning the value passed by PostQuitMessage(). 

        If this function must abort before entering the message loop, it 
        returns the conventional value NULL.  

****************************************************************************/

int PASCAL WinMain (hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                /* current instance             */
HANDLE hPrevInstance;            /* previous instance            */
LPSTR lpCmdLine;                 /* command line                 */
int nCmdShow;				     /* show-window type (open/icon) */
{
    MSG msg;                     /* message                      */

    if (!hPrevInstance)               /* Other instances of app running?   */
    if (!InitApplication (hInstance)) /* Initialize shared things          */
        return (FALSE);               /* Exits if unable to initialize     */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance (hInstance, nCmdShow))
        return (FALSE);

    if ((wDLLVersion = sbcGetDLLVersion ()) == NULL)
        return (FALSE);

    if ((lpCardName = sbcGetCardName ()) == NULL)
        return (FALSE);

    wSoundBlasterMsg = RegisterWindowMessage ((LPSTR) "SoundBlaster");

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage (&msg,     /* message structure                      */
        NULL,                    /* handle of window receiving the message */
        NULL,                    /* lowest message to examine              */
        NULL))                   /* highest message to examine             */
	{
        TranslateMessage (&msg); /* Translates virtual key codes           */
        DispatchMessage (&msg);  /* Dispatches message to window           */
    }

    sbcTerminateDLL ();

    return (msg.wParam);        /* Returns the value from PostQuitMessage */
}


/****************************************************************************

    FUNCTION: InitApplication (HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other 
        instances of the application are running.  This function performs 
        initialization tasks that can be done once for any number of running 
        instances.  

        In this case, we initialize a window class by filling out a data 
        structure of type WNDCLASS and calling the Windows RegisterClass() 
        function.  Since all instances of this application use the same window 
        class, we only need to do this when the first instance is initialized.  

****************************************************************************/

BOOL InitApplication (hInstance)
HANDLE hInstance;                       /* current instance                   */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style         = NULL;             /* Class style(s).                    */
    wc.lpfnWndProc   = MainWndProc;      /* Function to retrieve messages for  */
                                         /* windows of this class.             */
    wc.cbClsExtra    = 0;                /* No per-class extra data.           */
    wc.cbWndExtra    = 0;                /* No per-window extra data.          */
    wc.hInstance     = hInstance;        /* Application that owns the class.   */
    wc.hIcon         = LoadIcon (hInstance, "dllmusic");
    wc.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName  = "DLLMusicMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "DLLMusicWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

    return (RegisterClass (&wc));

}


/****************************************************************************

    FUNCTION:  InitInstance (HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

        This function is called at initialization time for every instance of 
        this application.  This function performs initialization tasks that 
        cannot be shared by multiple instances.  

        In this case, we save the instance handle in a static variable and 
        create and display the main program window.  
        
****************************************************************************/

BOOL InitInstance (hInstance, nCmdShow)
    HANDLE          hInstance;          /* Current instance identifier.       */
    int             nCmdShow;           /* Param for first ShowWindow() call. */
{
    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hWnd = CreateWindow
    (
        "DLLMusicWClass",               /* See RegisterClass() call.          */
        "DLLMUSIC Sample Application",  /* Text for window title bar.         */
        WS_OVERLAPPEDWINDOW,            /* Window style.                      */
        CW_USEDEFAULT,                  /* Default horizontal position.       */
        CW_USEDEFAULT,                  /* Default vertical position.         */
        CW_USEDEFAULT,                  /* Default width.                     */
        CW_USEDEFAULT,                  /* Default height.                    */
        NULL,                           /* Overlapped windows have no parent. */
        NULL,                           /* Use the window class menu.         */
        hInstance,                      /* This instance owns this window.    */
        NULL                            /* Pointer not needed.                */
    );

    /* If window could not be created, return "failure" */

    if (!hWnd)
        return (FALSE);

    /* Make the window visible; update its client area; and return "success" */

    ShowWindow (hWnd, nCmdShow);   /* Show the window                        */
    UpdateWindow (hWnd);           /* Sends WM_PAINT message                 */
    return (TRUE);                 /* Returns the value from PostQuitMessage */

}

/****************************************************************************

    FUNCTION: MainWndProc (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES:

	WM_COMMAND    - application menu (About dialog box)
	WM_DESTROY    - destroy window

    COMMENTS:

	To process the IDM_ABOUT message, call MakeProcInstance() to get the
	current instance address of the About() function.  Then call Dialog
	box which will create the box according to the information in your
    dllmusic.rc file and turn control over to the About() function.  When
	it returns, free the intance address.

****************************************************************************/

long FAR PASCAL MainWndProc (hWnd, message, wParam, lParam)
HWND hWnd;                    /* window handle                   */
unsigned message;             /* type of message                 */
WORD wParam;                  /* additional information          */
LONG lParam;                  /* additional information          */
{
    PAINTSTRUCT ps;
    FARPROC lpProcAbout;      /* pointer to the "About" function */

    switch (message)
    {
        case WM_PAINT:
            BeginPaint (hWnd, &ps);
            EndPaint (hWnd, &ps);
            DisplayMusicFlag (wMusicStatus);
            DisplayTransposeValue (wTransposeValue);
            DisplayTempoValue (wTempoValue);
            break;
        case WM_COMMAND:       /* message: command from application menu */
            switch (wParam)
            {
                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance (About, hInst);
                    DialogBox(hInst,         /* current instance         */
                        "AboutBox",          /* resource to use          */
                        hWnd,                /* parent handle            */
                        lpProcAbout);        /* About() instance address */
                    FreeProcInstance (lpProcAbout);
                    break;
                case IDM_START:
                    PlayMusicProcess ();
                    break;
                case IDM_STOP:
                    StopMusicProcess ();
                    break;
                case IDM_PAUSE:
                    PauseMusicProcess ();
                    break;
                case IDM_CONTINUE:
                    ContinueMusicProcess ();
                    break;
                case IDM_TRANSPOSE_UP:
                    TransposeMusic (1);
                    break;
                case IDM_TRANSPOSE_DOWN:
                    TransposeMusic (-1);
                    break;
                case IDM_TEMPO_UP:
                    ChangeMusicTempo (10);
                    break;
                case IDM_TEMPO_DOWN:
                    ChangeMusicTempo (-10);
                    break;
                case IDM_FADE_IN:
                    SetFading (100);
                    break;
                case IDM_FADE_OUT:
                    SetFading (0);
                    break;
                default:          /* Lets Windows process it         */
                    return (DefWindowProc (hWnd, message, wParam, lParam));
            }
            break;
        case WM_DESTROY:          /* message: window being destroyed */
            PostQuitMessage(0);
            break;

        default:
            if (message == wSoundBlasterMsg)
            {
                switch (wParam)
                {
                    case SBM_CALLBACK:
                        sbcCallBack (lParam);
                        break;
                    case SBM_MUS_FLAG:
                        switch (HIWORD (lParam))
                        {
                            case MP_PLAY_MEMMIDI:
                                wMusicStatus = LOWORD (lParam);
                                DisplayMusicFlag (wMusicStatus);
                                if (LOWORD (lParam) == 0)
                                {
                                    musCloseDevice ();
                                    ReleaseMemory ();
                                }
                                break;
                        }
                        break;
                }
            }
            else                  /* Passes it on if unproccessed    */
            {
                return (DefWindowProc(hWnd, message, wParam, lParam));
            }
    }
    return (NULL);
}

/****************************************************************************

    FUNCTION: LoadMusicFile ()

    PURPOSE: Load a midi file into memory

****************************************************************************/

HANDLE LoadMusicFile ()
{
	HANDLE	    hFile ;
    OFSTRUCT    os;
    LONG        lFileLen ;
    HANDLE      hMusicMem;
    LPSTR       lpMidiBuffer;

    if (hFile = OpenFile ("demo.mid", &os,OF_READ))
    {
        lFileLen = _llseek (hFile, 0L, 2) ;
        _llseek (hFile, 0L, 0);
        hMusicMem = GlobalAlloc (GMEM_MOVEABLE, lFileLen);
        if (lpMidiBuffer = GlobalLock (hMusicMem))
        {
            _lread (hFile, lpMidiBuffer, 0xFFFF) ;
            GlobalUnlock (hMusicMem);
            wTempoValue = 100;
            DisplayTempoValue (wTempoValue);
            wTransposeValue = 0;
            DisplayTransposeValue (wTransposeValue);
        }
        else
        {
            MessageBeep (0);
            MessageBox (hWnd, "Cannot allocate memory", "Error...",
                        MB_ICONEXCLAMATION | MB_OK);
        }
        _lclose (hFile);
    }
    else
    {
        hMusicMem = NULL;
        MessageBeep (0);
        MessageBox (hWnd, "Cannot open music file", "Error...",
                    MB_ICONEXCLAMATION | MB_OK);
    }
    return hMusicMem;
}

/****************************************************************************

    FUNCTION: ReleaseMemory ()

    PURPOSE: Return allocated memory block to windows

****************************************************************************/

void ReleaseMemory ()
{
    GlobalPageUnlock (HIWORD (lpBuffer));
    GlobalUnWire (hMusicMem);
    GlobalFree (hMusicMem);
}

/****************************************************************************

    FUNCTION: PlayMusicProcess ()

    PURPOSE: Start music playback

****************************************************************************/

void PlayMusicProcess ()
{
    WORD wReturnCode;

    if (wReturnCode = musOpenDevice (hWnd))
        DisplayErrorCode (wReturnCode);
    else
    {
        if (hMusicMem = LoadMusicFile ())
        {
            lpBuffer = GlobalWire (hMusicMem);
            GlobalPageLock (HIWORD (lpBuffer));
            if (wReturnCode = musPlayMemMidi (lpBuffer))
            {
                musCloseDevice ();
                ReleaseMemory ();
                DisplayErrorCode (wReturnCode);
            }
        }
    }
}

/****************************************************************************

    FUNCTION: StopMusicProcess ()

    PURPOSE: Stop current active music process

****************************************************************************/

void StopMusicProcess ()
{
    WORD wReturnValue;

    if (wReturnValue = musStopMusic ())
        DisplayErrorCode (wReturnValue);
}

/****************************************************************************

    FUNCTION: PauseMusicProcess ()

    PURPOSE: Pause current active music process

****************************************************************************/

void PauseMusicProcess ()
{
    WORD wReturnValue;

    if (wReturnValue = musPauseMusic ())
        DisplayErrorCode (wReturnValue);
}

/****************************************************************************

    FUNCTION: ContinueMusicProcess ()

    PURPOSE: Continue previously paused music process

****************************************************************************/

void ContinueMusicProcess ()
{
    WORD wReturnValue;

    if (wReturnValue = musContinueMusic ())
        DisplayErrorCode (wReturnValue);
}

/****************************************************************************

    FUNCTION: DisplayErrorCode (WORD)

    PURPOSE: Display error code

****************************************************************************/

void DisplayErrorCode (error)
WORD error;
{
    char szTmpBuf[20];

    wsprintf (szTmpBuf, "Error code = %d", error);
    MessageBeep (0);
    MessageBox (NULL, szTmpBuf, "Error...", MB_ICONINFORMATION | MB_OK);
}

/****************************************************************************

    FUNCTION: DisplayMusicFlag (WORD)

    PURPOSE: Display status of music process

****************************************************************************/

void DisplayMusicFlag (mus_status)
WORD mus_status;
{
    HDC hDC;
    int x, y;
    char szText[64];

    if (mus_status)
    {
        EnableMenuItem (GetMenu (hWnd), IDM_START, MF_DISABLED | MF_GRAYED);
        EnableMenuItem (GetMenu (hWnd), IDM_STOP, MF_ENABLED);
    }
    else
    {
        EnableMenuItem (GetMenu (hWnd), IDM_START, MF_ENABLED);
        EnableMenuItem (GetMenu (hWnd), IDM_STOP, MF_DISABLED | MF_GRAYED);
    }
    wsprintf (szText, "Music Flag           = %04x    ", mus_status);
    hDC = GetDC (hWnd);
    x = GetDeviceCaps (hDC, LOGPIXELSX)*2.75;
    y = GetDeviceCaps (hDC, LOGPIXELSY)*0.25;
    TextOut (hDC, x, y, szText, strlen (szText));
    ReleaseDC (hWnd, hDC);
}

/****************************************************************************

    FUNCTION: ChangeMusicTempo (int)

    PURPOSE: Increase or decrease tempo

****************************************************************************/

void ChangeMusicTempo (wChangeValue)
int wChangeValue;
{
    wTempoValue += wChangeValue;

    if (wTempoValue > 400)
        wTempoValue = 400;
    else
    {
        if (wTempoValue < 25)
            wTempoValue = 25;
    }

    DisplayTempoValue (wTempoValue);
    musChangeTempo (wTempoValue);
}

/****************************************************************************

    FUNCTION: DisplayTempoValue (int)

    PURPOSE: Display tempo value

****************************************************************************/

void DisplayTempoValue (wTempoValue)
int wTempoValue;
{
    HDC hDC;
    int x, y;
    char szText[64];

    wsprintf (szText, "Tempo value        = %d    ", wTempoValue);
    hDC = GetDC (hWnd);
    x = GetDeviceCaps (hDC, LOGPIXELSX)*2.75;
    y = GetDeviceCaps (hDC, LOGPIXELSY)*0.65;
    TextOut (hDC, x, y, szText, strlen (szText));
    ReleaseDC (hWnd, hDC);
}

/****************************************************************************

    FUNCTION: TransposeMusic (int)

    PURPOSE: Transpose melody up or down

****************************************************************************/

void TransposeMusic (wSemiTone)
int wSemiTone;
{
    wTransposeValue += wSemiTone;

    if (wTransposeValue > 24)
        wTransposeValue = 24;
    else
    {
        if (wTransposeValue < -24)
            wTransposeValue = -24;
    }

    DisplayTransposeValue (wTransposeValue);
    musTransposeNote (wTransposeValue);
}

/****************************************************************************

    FUNCTION: DisplayTransposeValue (int)

    PURPOSE: Display transpose value

****************************************************************************/

void DisplayTransposeValue (wTransposeValue)
int wTransposeValue;
{
    HDC hDC;
    int x, y;
    char szText[64];

    wsprintf (szText, "Transpose value  = %d    ", wTransposeValue);
    hDC = GetDC (hWnd);
    x = GetDeviceCaps (hDC, LOGPIXELSX)*2.75;
    y = GetDeviceCaps (hDC, LOGPIXELSY)*0.45;
    TextOut (hDC, x, y, szText, strlen (szText));
    ReleaseDC (hWnd, hDC);
}

/****************************************************************************

    FUNCTION: SetFading (WORD)

    PURPOSE: Increase or decrease volume in steps

****************************************************************************/

void SetFading (wFinalValue)
WORD wFinalValue;
{
    WORD wReturnCode;

    if (wReturnCode = musFadeMusic (-1, wFinalValue, 5, 5))
        DisplayErrorCode (wReturnCode);
}

/****************************************************************************

    FUNCTION: About (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL About (hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{
    HDC     hDC;
    HDC     hMemoryDC;
    char    szTmpBuf[128];
    BITMAP  Bitmap;
    HBITMAP hBmSBLogo;
    HBITMAP hOldBitmap;

    switch (message)
    {
        case WM_INITDIALOG:           /* message: initialize dialog box */
            wsprintf (szTmpBuf, "Hardware Card: %s", lpCardName);
            SetDlgItemText (hDlg, IDT_CARDNAME, szTmpBuf);
            wsprintf (szTmpBuf, "DLL Version: %d.%02d", HIBYTE (wDLLVersion),
                                                        LOBYTE (wDLLVersion));
            SetDlgItemText (hDlg, IDT_DLLVERSION, szTmpBuf);
            return (TRUE);

        case WM_COMMAND:                      /* message: received a command */
            if (wParam == IDOK                /* "OK" box selected?          */
                    || wParam == IDCANCEL)    /* System menu close command?  */
            {
                EndDialog (hDlg, TRUE);       /* Exits the dialog box        */
                return (TRUE);
            }
            break;
        case WM_PAINT:
            hBmSBLogo = LoadBitmap (hInst, "sbct");
            GetObject (hBmSBLogo, sizeof (BITMAP), (LPSTR) &Bitmap);
            hDC = GetDC (hDlg);
            hMemoryDC = CreateCompatibleDC (hDC);
            hOldBitmap = SelectObject (hMemoryDC, hBmSBLogo);
            if (hOldBitmap)
            {
                BitBlt (hDC, 3, 3, Bitmap.bmWidth, Bitmap.bmHeight,
                        hMemoryDC, 0, 0, SRCCOPY);
                SelectObject (hMemoryDC, hOldBitmap);
            }
            DeleteDC (hMemoryDC);
            ReleaseDC (hDlg, hDC);
            DeleteObject (hBmSBLogo);
            break;
    }
    return (FALSE);			      /* Didn't process a message    */
}
